/*
 * 
 * Name:   ZNodeConfigClient
 * 
 * Author: Bhaskar S
 * 
 * Date:   05/31/2014
 * 
 */

package com.polarsparc.zookeeper;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.zookeeper.ZooKeeper;
import org.apache.zookeeper.Watcher;
import org.apache.zookeeper.WatchedEvent;

// ----- Usage -----
// java com.polarsparc.zookeeper.ZNodeConfigClient <host-port-list>
//
public class ZNodeConfigClient {
    private static final int SESSION_TIMEOUT = 10000;
    private static final String path = "/ConfigMgmt/config";
    
    private static final Logger log = LoggerFactory.getLogger(ZNodeConfigClient.class);
    
    // ----- Main Method -----
    
    public static void main(String[] args) {
        if (args.length != 1) {
            System.out.printf("Specify <host-port-list>\n");
            System.exit(1);
        }
        
        try {
            ZooKeeper zk = ZkHelper.connect(args[0], SESSION_TIMEOUT);
            
            boolean status = ZkHelper.checkZnode(zk, path, new MyWatcher(zk));
            
            log.info("Check ZNode " + path + ", Status = " + status);
            
            if (status == true) {
                String str = ZkHelper.getDataZnode(zk, path);

                log.info("Get ZNode " + path + ", Data = " + str);
            }
            
            log.info("Ready to sleep for 60 seconds !!!");
            
            Thread.sleep(60 * 1000); // 60 Seconds
            
            ZkHelper.close(zk);
            
            log.info("Done !!!");
        }
        catch (Exception ex) {
            log.error("ZNodeConfigClient.main(): Exception", ex);
        }
    }
    
    static class MyWatcher implements Watcher {
        private final ZooKeeper zk;
        
        MyWatcher(ZooKeeper zk) {
            this.zk = zk;
        }
        
        @Override
        public void process(WatchedEvent event) {
            log.info("-----> Notification for ZNode " + event.getPath() + " <-----");
            
            if (event.getType() == Watcher.Event.EventType.NodeDataChanged) {
                String str = ZkHelper.getDataZnode(zk, ZNodeConfigClient.path);
            
                log.info("Get ZNode " + ZNodeConfigClient.path + ", Data = " + str);
            }
        }
    }
}
