/*
 * 
 * Name:   ZNodeConfigSetup
 * 
 * Author: Bhaskar S
 * 
 * Date:   05/31/2014
 * 
 */

package com.polarsparc.zookeeper;

import java.io.File;
import java.io.FileInputStream;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.ZooKeeper;

// ----- Usage -----
// java com.polarsparc.zookeeper.ZNodeConfigSetup <config-file> <host-port-list>
//
public class ZNodeConfigSetup {
    private static final int SESSION_TIMEOUT = 10000;
    private static final String path1 = "/ConfigMgmt";
    private static final String path2 = "/ConfigMgmt/config";
    private static final byte[] data1 = "".getBytes();
    
    private static final Logger log = LoggerFactory.getLogger(ZNodeConfigSetup.class);
    
    // ----- Main Method -----
    
    public static void main(String[] args) {
        if (args.length != 2) {
            System.out.printf("Specify <config-file> <host-port-list>\n");
            System.exit(1);
        }
        
        try {
            byte[] data2 = null;
            
            File file = new File(args[0]);
            
            try (FileInputStream fin = new FileInputStream(file)) {
                data2 = new byte[(int)file.length()];
                fin.read(data2);
            }
            catch (Exception ex) {
                log.error("ZNodeConfigSetup.main(): Exception", ex);
            }
            
            ZooKeeper zk = ZkHelper.connect(args[1], SESSION_TIMEOUT);
            
            boolean status = ZkHelper.checkZnode(zk, path1, null);
            
            log.info("Check ZNode " + path1 + ", Status = " + status);
            
            if (status == false) {
                ZkHelper.createZnode(zk, CreateMode.PERSISTENT, path1, data1);
            }
            
            status = ZkHelper.checkZnode(zk, path2, null);
            
            log.info("Check ZNode " + path2 + ", Status = " + status);
            
            if (status == false) {
                ZkHelper.createZnode(zk, CreateMode.PERSISTENT, path2, data2);
            }
            
            String str = ZkHelper.getDataZnode(zk, path2);
            
            log.info("Get ZNode " + path2 + ", Data = " + str);
            
            ZkHelper.close(zk);
        }
        catch (Exception ex) {
            log.error("ZNodeConfigSetup.main(): Exception", ex);
        }
    }
}
