/*
 * 
 * Name:   ZNodeConfigUpdate
 * 
 * Author: Bhaskar S
 * 
 * Date:   05/31/2014
 * 
 */

package com.polarsparc.zookeeper;

import java.io.File;
import java.io.FileInputStream;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.zookeeper.ZooKeeper;

// ----- Usage -----
// java com.polarsparc.zookeeper.ZNodeConfigUpdate <config-file> <host-port-list>
//
public class ZNodeConfigUpdate {
    private static final int SESSION_TIMEOUT = 10000;
    private static final String path = "/ConfigMgmt/config";
    
    private static final Logger log = LoggerFactory.getLogger(ZNodeConfigUpdate.class);
    
    // ----- Main Method -----
    
    public static void main(String[] args) {
        if (args.length != 2) {
            System.out.printf("Specify <config-file> <host-port-list>\n");
            System.exit(1);
        }
        
        try {
            byte[] data = null;
            
            File file = new File(args[0]);
            
            try (FileInputStream fin = new FileInputStream(file)) {
                data = new byte[(int)file.length()];
                fin.read(data);
            }
            catch (Exception ex) {
                log.error("ZNodeConfigUpdate.main(): Exception", ex);
            }
            
            ZooKeeper zk = ZkHelper.connect(args[1], SESSION_TIMEOUT);
            
            boolean status = ZkHelper.checkZnode(zk, path, null);
            
            log.info("Check ZNode " + path + ", Status = " + status);
            
            if (status == true) {
                status = ZkHelper.setDataZnode(zk, path, data);

                log.info("Set ZNode Data Status = " + status);
            }
            
            ZkHelper.close(zk);
        }
        catch (Exception ex) {
            log.error("ZNodeConfigSetup.main(): Exception", ex);
        }
    }    
}
