/*
 * 
 * Name:   ZNodeDistLock
 * 
 * Author: Bhaskar S
 * 
 * Date:   05/31/2014
 * 
 */

package com.polarsparc.zookeeper;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.ZooKeeper;
import org.apache.zookeeper.Watcher;
import org.apache.zookeeper.WatchedEvent;

// ----- Usage -----
// java com.polarsparc.zookeeper.ZNodeDistLock <unique-name> <count> <delay> <host-port-list>
//
public class ZNodeDistLock {
    private static final int SESSION_TIMEOUT = 10000;
    private static final String parent = "/DistLock";
    private static final String lock = "/DistLock/lock";
    private static final byte[] data = "1".getBytes();
    private static final byte[] data1 = "".getBytes();
    
    private static final Logger log = LoggerFactory.getLogger(ZNodeDistLock.class);
    
    private static long delay = 0;
    
    private static String client = null;
    
    // ----- Main Method -----
    
    public static void main(String[] args) {
        if (args.length != 4) {
            System.out.printf("Specify <unique-name> <count> <delay> <host-port-list>\n");
            System.exit(1);
        }
        
        try {
            client = args[0];
            int count = Integer.parseInt(args[1]);
            delay = Long.parseLong(args[2]);
            
            ZooKeeper zk = ZkHelper.connect(args[3], SESSION_TIMEOUT);
            
            boolean status = ZkHelper.checkZnode(zk, parent, null);
            
            log.info("Check ZNode " + parent + ", Status = " + status);
            
            if (status == false) {
                ZkHelper.createZnode(zk, CreateMode.PERSISTENT, parent, data);
            }
            
            status = ZkHelper.checkZnode(zk, parent, null);
            
            if (status == true) {
                for (int i = 0; i < count; i++) {
                    lock(zk);
                    
                    String str = ZkHelper.getDataZnode(zk, parent);
                    
                    log.info("Client " + client + " processing [" + str + "]");
                    
                    int x = Integer.parseInt(str);
                    
                    byte[] data2 = (Integer.toString(x+1)).getBytes();
                    
                    ZkHelper.setDataZnode(zk, parent, data2);
                    
                    unlock(zk);
                    
                    Thread.sleep(delay);
                }
            }
            
            ZkHelper.close(zk);
        }
        catch (Exception ex) {
            log.error("ZNodeDistLock.main(): Exception", ex);
        }
    }
    
    static void lock(ZooKeeper zk) {
        final Object mutex = new Object();
        
        Watcher watcher = new MyLockWatcher(mutex);
        
        for (;;) {
            List<String> list = ZkHelper.getZnodeChildren(zk, parent, watcher);
            if (list.isEmpty()) {
                // Try to grab znode
                boolean status = ZkHelper.createZnode(zk, CreateMode.EPHEMERAL, lock, data1);
                if (status) {
                    // Success
                    log.info("Client " + client + " acquired lock !!!");
                    break;
                }
                else {
                    log.info("Ouch, client " + client + " missed ...");
                    try {
                        synchronized (mutex) {
                            mutex.wait(delay);
                        }
                    }
                    catch (InterruptedException ex) {
                    }
                }
            }
            else {
                log.info("Client " + client + " have to wait ...");
                try {
                    synchronized (mutex) {
                        mutex.wait(delay);
                    }
                }
                catch (InterruptedException ex) {
                }
            }
        }
    }
    
    static void unlock(ZooKeeper zk) {
        boolean status = ZkHelper.deleteZnode(zk, lock);
        if (status) {
            log.info("Client " + client + " released lock !!!");
        }
    }
    
    static class MyLockWatcher implements Watcher {
        private final Object mutex;
        
        MyLockWatcher(Object mutex) {
            this.mutex = mutex;
        }
        
        @Override
        public void process(WatchedEvent event) {
            log.info("-----> Notification for ZNode " + event.getPath() + " <-----");
            
            if (event.getType() == Watcher.Event.EventType.NodeDataChanged) {
                synchronized (mutex) {
                    mutex.notifyAll();
                }
            }
        }
    }
}
