/*
 * 
 * Name:   ZNodeGroupMember
 * 
 * Author: Bhaskar S
 * 
 * Date:   05/31/2014
 * 
 */

package com.polarsparc.zookeeper;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.ZooKeeper;

// ----- Usage -----
// java com.polarsparc.zookeeper.ZNodeGroupMember <unique-name> <delay> <host-port-list>
//
public class ZNodeGroupMember {
    private static final int SESSION_TIMEOUT = 10000;
    private static final String SLASH = "/";
    private static final String parent = "/GroupMember";
    private static final byte[] data = "".getBytes();
    
    private static final Logger log = LoggerFactory.getLogger(ZNodeGroupMember.class);
    
    // ----- Main Method -----
    
    public static void main(String[] args) {
        if (args.length != 3) {
            System.out.printf("Specify <unique-name> <delay> <host-port-list>\n");
            System.exit(1);
        }
        
        try {
            String path = parent + SLASH + args[0];
            
            long delay = Long.parseLong(args[1]);
            
            ZooKeeper zk = ZkHelper.connect(args[2], SESSION_TIMEOUT);
            
            boolean status = ZkHelper.checkZnode(zk, parent, null);
            
            log.info("Check ZNode " + parent + ", Status = " + status);
            
            if (status == false) {
                ZkHelper.createZnode(zk, CreateMode.PERSISTENT, parent, data);
            }
            
            log.info("Client member ZNode = " + path);
            
            status = ZkHelper.checkZnode(zk, path, null);
            
            log.info("Check ZNode " + path + ", Status = " + status);
            
            if (status == false) {
                ZkHelper.createZnode(zk, CreateMode.EPHEMERAL, path, data);
            }
            
            Thread.sleep(delay);
            
            ZkHelper.close(zk);
        }
        catch (Exception ex) {
            log.error("ZNodeGroupMember.main(): Exception", ex);
        }
    }
}
