/*
 * 
 * Name:   ZNodeGroupMonitor
 * 
 * Author: Bhaskar S
 * 
 * Date:   05/31/2014
 * 
 */

package com.polarsparc.zookeeper;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.ZooKeeper;

// ----- Usage -----
// java com.polarsparc.zookeeper.ZNodeGroupMonitor <count> <delay> <host-port-list>
//
public class ZNodeGroupMonitor {
    private static final int SESSION_TIMEOUT = 10000;
    private static final String parent = "/GroupMember";
    private static final byte[] data = "".getBytes();
    
    private static final Logger log = LoggerFactory.getLogger(ZNodeGroupMonitor.class);
    
    // ----- Main Method -----
    
    public static void main(String[] args) {
        if (args.length != 3) {
            System.out.printf("Specify <count> <delay> <host-port-list>\n");
            System.exit(1);
        }
        
        try {
            int count = Integer.parseInt(args[0]);
            
            long delay = Long.parseLong(args[1]);
            
            ZooKeeper zk = ZkHelper.connect(args[2], SESSION_TIMEOUT);
            
            boolean status = ZkHelper.checkZnode(zk, parent, null);
            
            log.info("Check ZNode " + parent + ", Status = " + status);
            
            if (status == false) {
                ZkHelper.createZnode(zk, CreateMode.PERSISTENT, parent, data);
            }
            
            status = ZkHelper.checkZnode(zk, parent, null);
            
            if (status == true) {
                for (int i = 0; i < count; i++) {
                    List<String> list = ZkHelper.getZnodeChildren(zk, parent, null);
                    if (list != null) {
                        log.info("Members = " + list);
                    }
                    
                    Thread.sleep(delay);
                }
            }
            
            ZkHelper.close(zk);
        }
        catch (Exception ex) {
            log.error("ZNodeGroupMonitor.main(): Exception", ex);
        }
    }
}
