/*
 * 
 * Name:   ZNodeTests
 * 
 * Author: Bhaskar S
 * 
 * Date:   05/24/2014
 * 
 */

package com.polarsparc.zookeeper;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.ZooKeeper;
import org.apache.zookeeper.Watcher;
import org.apache.zookeeper.WatchedEvent;

// ----- Usage -----
// java com.polarsparc.zookeeper.ZNodeTests <host-port-list>
//
public class ZNodeTests {
    private static final int SESSION_TIMEOUT = 10000;
    private static final String ppath = "/mytest";
    private static final String spath1 = "/mytest/child-1";
    private static final String spath2 = "/mytest/child-2";
    private static final String tpath = "/mytmp";
    private static final byte[] data = "".getBytes();
    private static final byte[] data2 = "hello".getBytes();
    
    private static final Logger log = LoggerFactory.getLogger(ZNodeTests.class);
    
    // ----- Main Method -----
    
    public static void main(String[] args) {
        if (args.length != 1) {
            System.out.printf("Specify <host-port-list>\n");
            System.exit(1);
        }
        
        try {
            ZooKeeper zk = ZkHelper.connect(args[0], SESSION_TIMEOUT);
            
            boolean status = ZkHelper.createZnode(zk, CreateMode.PERSISTENT, ppath, data);
            
            log.info("Create ZNode " + ppath + " Status = " + status);
            
            status = ZkHelper.checkZnode(zk, ppath, (WatchedEvent event) -> {
                if (event.getType() == Watcher.Event.EventType.NodeDataChanged) {
                    log.info("-----> Data at znode " + event.getPath() + " changed <-----");
                }
            });
            
            log.info("Check ZNode " + ppath + " Status = " + status);
            
            ZkHelper.createZnode(zk, CreateMode.PERSISTENT, spath1, data);
            ZkHelper.createZnode(zk, CreateMode.PERSISTENT, spath2, data);
            
            List<String> children = ZkHelper.getZnodeChildren(zk, ppath, null);
            
            log.info("Get ZNode " + ppath + " Children = " + children);
            
            String str = ZkHelper.getDataZnode(zk, ppath);
            
            log.info("Get ZNode " + ppath + " Data = " + str);
            
            status = ZkHelper.setDataZnode(zk, ppath, data2);
            
            log.info("Set ZNode Data " + ppath + " Status = " + status);
            
            status = ZkHelper.deleteZnode(zk, spath2);
            
            log.info("Delete ZNode " + spath2 + " Status = " + status);
            
            status = ZkHelper.deleteZnode(zk, spath1);
            
            log.info("Delete ZNode " + spath1 + " Status = " + status);
            
            status = ZkHelper.deleteZnode(zk, ppath);
            
            log.info("Delete ZNode " + ppath + " Status = " + status);
            
            status = ZkHelper.createZnode(zk, CreateMode.EPHEMERAL, tpath, data);
            
            log.info("Create ZNode " + tpath + " Status = " + status);
            
            ZkHelper.close(zk);
        }
        catch (Exception ex) {
            log.error("ZNodeTests.main(): Exception", ex);
        }
    }
}
